@@ -19,8 +19,10 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
+
 #include "mainwin.h"
 
+#include <QDockWidget>
 #include <QPushButton>
 #include <QLabel>
 #include <QDebug>
@@ -76,6 +78,7 @@ MainWindow::setupActions()
     KStandardGameAction::quit( this, SLOT( close() ), actionCollection() );
 
     m_endTurnAction = KStandardGameAction::endTurn(this, NULL, actionCollection());
+    m_endTurnAction->setShortcut(Qt::CTRL + Qt::Key_E);
     m_endTurnAction->setEnabled(false);
 
     m_endGameAction = KStandardGameAction::end( this, NULL, actionCollection() );
@@ -88,12 +91,6 @@ MainWindow::setupActions()
     m_measureAction->setText( i18n("&Measure Distance") );
     m_measureAction->setEnabled(false);
 
-    // Show standings
-    m_standingAction = actionCollection()->addAction( QLatin1String(  "game_scores" ) );
-    m_standingAction->setIcon( KIcon( QLatin1String( "help-contents" )) );
-    m_standingAction->setText( i18n("&Show Standings") );
-    m_standingAction->setEnabled(false);
-
     // Show fleet overview
     m_fleetAction = actionCollection()->addAction( QLatin1String(  "game_fleets" ) );
     m_fleetAction->setIcon( KIcon( QLatin1String( "fork" )) );
@@ -103,6 +100,47 @@ MainWindow::setupActions()
     // Toolbar
     addToolBar ( Qt::LeftToolBarArea, toolBar() );
     toolBar()->setMovable(false);
+
+    // docking area - messages
+
+    m_messagesDock = new QDockWidget(i18n("Messages"), this);
+    m_messagesDock->setObjectName("dock-messages");
+
+    addDockWidget(Qt::BottomDockWidgetArea, m_messagesDock);
+
+    m_messagesAction = actionCollection()->addAction(QLatin1String("view_messages"));
+    m_messagesAction->setText(i18n("Show &Messages"));
+    m_messagesAction->setCheckable(true);
+    m_messagesAction->setChecked(m_messagesDock->isVisible());
+
+    // The action signal "toggled" is fired even in case the state is changed
+    // via code using setChecked(). "triggered" however is only fired if the
+    // user actually triggered the change.
+
+    // The dock signal "visibilityChanged" is fired if the dock is shown or
+    // hidden. But this includes hidden in a tab as well. The action should not
+    // represent the visibility state, but if the dock is present somewhere in
+    // the GUI, regardless of currently visible in an active tab or invisible
+    // in a not currently active tab.
+
+    connect(m_messagesAction, SIGNAL(triggered(bool)), m_messagesDock, SLOT(setVisible(bool)));
+    connect(m_messagesDock, SIGNAL(visibilityChanged(bool)), this, SLOT(updateMessagesActionSlot()));
+
+    // docking area - standings
+
+    m_standingsDock = new QDockWidget(i18n("Standings"), this);
+    m_standingsDock->setObjectName("dock-standings");
+
+    tabifyDockWidget(m_messagesDock, m_standingsDock);
+
+    m_standingsAction = actionCollection()->addAction(QLatin1String("view_standings"));
+    m_standingsAction->setIcon(KIcon(QLatin1String("help-contents")));
+    m_standingsAction->setText(i18n("Show &Standings"));
+    m_standingsAction->setCheckable(true);
+    m_standingsAction->setChecked(m_standingsDock->isVisible());
+
+    connect(m_standingsAction, SIGNAL(triggered(bool)), m_standingsDock, SLOT(setVisible(bool)));
+    connect(m_standingsDock, SIGNAL(visibilityChanged(bool)), this, SLOT(updateStandingsActionSlot()));
 }
 
 
@@ -110,7 +148,7 @@ void
 MainWindow::setupGameView()
 {
     m_game      = new LocalGame( this );
-    m_gameView  = new GameView( this, m_game );
+    m_gameView  = new GameView(this, m_game, m_messagesDock, m_standingsDock);
     setCentralWidget( m_gameView );
 
     connect ( m_game,    SIGNAL( gameMsg(const KLocalizedString &,
@@ -123,12 +161,37 @@ MainWindow::setupGameView()
 	     this,       SLOT( guiStateChange( GUIState ) ) );
 
     connect(m_measureAction,  SIGNAL(triggered(bool)), m_gameView, SLOT( measureDistance() ));
-    connect(m_standingAction, SIGNAL(triggered(bool)), m_gameView, SLOT( showScores() ));
     connect(m_fleetAction,    SIGNAL(triggered(bool)), m_gameView, SLOT( showFleets() ));
     connect(m_endTurnAction,  SIGNAL(triggered()),     m_gameView, SLOT(nextPlayer()));
     connect(m_endGameAction,  SIGNAL(triggered()),     m_gameView, SLOT(shutdownGame()));
 }
 
+
+void
+MainWindow::setupGUI()
+{
+    KXmlGuiWindow::setupGUI();
+
+    /**
+     * @todo The docks should not be visible on the main screen, and neither
+     * should it be possible to open the docks. During the game and later on,
+     * this is handled by GameView::changeGameView() and by
+     * MainWindow::guiStateChange(). Just the initial setup does not work that
+     * way. - Rework the GUI setup sequence so that the following hack is not
+     * required.
+     */
+
+    m_messagesAction->setEnabled(false);
+    m_standingsAction->setEnabled(false);
+
+    m_messagesDock->toggleViewAction()->setEnabled(false);
+    m_standingsDock->toggleViewAction()->setEnabled(false);
+
+    m_messagesDock->hide();
+    m_standingsDock->hide();
+}
+
+
 void
 MainWindow::startNewGame()
 {
@@ -144,18 +207,47 @@ MainWindow::startNewGame()
 void
 MainWindow::guiStateChange( GUIState newState )
 {
-    if (newState == NONE)
-    {
+    if (newState == NONE) {
         m_gameView->deleteLater();
         m_game->deleteLater();
         this->setupGameView();
     }
 
-    m_endTurnAction ->setEnabled( m_game->isRunning() && (newState == SOURCE_PLANET));
+    // An alternative to disabling the "end turn" action during "send fleet
+    // command sequence" is to simply abort this sequence if the user decides
+    // to "end turn" before completion.
+
+    /**
+     * @todo The game view handles the state of the actions, so the game view
+     * should be able to update the enabled state of the actions as well. This
+     * should be implemented via signals, instead of copying the conditions here
+     * again.
+     */
+
+    m_endTurnAction ->setEnabled( m_game->isRunning() && (newState == SOURCE_PLANET) );
     m_endGameAction ->setEnabled( m_game->isRunning() );
     m_measureAction ->setEnabled( newState == SOURCE_PLANET );
-    m_standingAction->setEnabled( newState == SOURCE_PLANET );
     m_fleetAction   ->setEnabled( newState == SOURCE_PLANET );
 
+    m_messagesAction->setEnabled(m_game->isRunning());
+    m_standingsAction->setEnabled(m_game->isRunning());
+
+    m_messagesDock->toggleViewAction()->setEnabled(m_game->isRunning());
+    m_standingsDock->toggleViewAction()->setEnabled(m_game->isRunning());
+
     m_statusBarText->setText(i18n("Turn # %1", m_game->turnCounter()));
 }
+
+
+void
+MainWindow::updateMessagesActionSlot()
+{
+    m_messagesAction->setChecked(m_messagesDock->toggleViewAction()->isChecked());
+}
+
+
+void
+MainWindow::updateStandingsActionSlot()
+{
+    m_standingsAction->setChecked(m_standingsDock->toggleViewAction()->isChecked());
+}
